@@ -1,9 +1,9 @@
-use crate::error::{IntErr, Interrupt};
+use crate::error::{FendError, Interrupt};
 use crate::eval::evaluate_to_value;
 use crate::ident::Ident;
 use crate::interrupt::test_int;
 use crate::num::{Base, FormattingStyle, Number};
-use crate::scope::{GetIdentError, Scope};
+use crate::scope::Scope;
 use crate::value::{ApplyMulHandling, BuiltInFunction, Value};
 use std::sync::Arc;
 
@@ -40,39 +40,43 @@ pub(crate) enum Expr {
 }
 
 impl<'a> Expr {
-    pub(crate) fn format<I: Interrupt>(&self, int: &I) -> Result<String, IntErr<String, I>> {
+    pub(crate) fn format<I: Interrupt>(
+        &self,
+        ctx: &crate::Context,
+        int: &I,
+    ) -> Result<String, FendError> {
         Ok(match self {
             Self::Literal(Value::String(s)) => format!(r#""{}""#, s.as_ref()),
-            Self::Literal(v) => v.format_to_plain_string(0, int)?,
+            Self::Literal(v) => v.format_to_plain_string(0, ctx, int)?,
             Self::Ident(ident) => ident.to_string(),
-            Self::Parens(x) => format!("({})", x.format(int)?),
-            Self::UnaryMinus(x) => format!("(-{})", x.format(int)?),
-            Self::UnaryPlus(x) => format!("(+{})", x.format(int)?),
-            Self::UnaryDiv(x) => format!("(/{})", x.format(int)?),
-            Self::Factorial(x) => format!("{}!", x.format(int)?),
+            Self::Parens(x) => format!("({})", x.format(ctx, int)?),
+            Self::UnaryMinus(x) => format!("(-{})", x.format(ctx, int)?),
+            Self::UnaryPlus(x) => format!("(+{})", x.format(ctx, int)?),
+            Self::UnaryDiv(x) => format!("(/{})", x.format(ctx, int)?),
+            Self::Factorial(x) => format!("{}!", x.format(ctx, int)?),
             Self::Add(a, b) | Self::ImplicitAdd(a, b) => {
-                format!("({}+{})", a.format(int)?, b.format(int)?)
+                format!("({}+{})", a.format(ctx, int)?, b.format(ctx, int)?)
             }
-            Self::Sub(a, b) => format!("({}-{})", a.format(int)?, b.format(int)?),
-            Self::Mul(a, b) => format!("({}*{})", a.format(int)?, b.format(int)?),
-            Self::Div(a, b) => format!("({}/{})", a.format(int)?, b.format(int)?),
-            Self::Mod(a, b) => format!("({} mod {})", a.format(int)?, b.format(int)?),
-            Self::Pow(a, b) => format!("({}^{})", a.format(int)?, b.format(int)?),
-            Self::Apply(a, b) => format!("({} ({}))", a.format(int)?, b.format(int)?),
+            Self::Sub(a, b) => format!("({}-{})", a.format(ctx, int)?, b.format(ctx, int)?),
+            Self::Mul(a, b) => format!("({}*{})", a.format(ctx, int)?, b.format(ctx, int)?),
+            Self::Div(a, b) => format!("({}/{})", a.format(ctx, int)?, b.format(ctx, int)?),
+            Self::Mod(a, b) => format!("({} mod {})", a.format(ctx, int)?, b.format(ctx, int)?),
+            Self::Pow(a, b) => format!("({}^{})", a.format(ctx, int)?, b.format(ctx, int)?),
+            Self::Apply(a, b) => format!("({} ({}))", a.format(ctx, int)?, b.format(ctx, int)?),
             Self::ApplyFunctionCall(a, b) | Self::ApplyMul(a, b) => {
-                format!("({} {})", a.format(int)?, b.format(int)?)
+                format!("({} {})", a.format(ctx, int)?, b.format(ctx, int)?)
             }
-            Self::As(a, b) => format!("({} as {})", a.format(int)?, b.format(int)?),
+            Self::As(a, b) => format!("({} as {})", a.format(ctx, int)?, b.format(ctx, int)?),
             Self::Fn(a, b) => {
                 if a.as_str().contains('.') {
-                    format!("({}:{})", a, b.format(int)?)
+                    format!("({}:{})", a, b.format(ctx, int)?)
                 } else {
-                    format!("\\{}.{}", a, b.format(int)?)
+                    format!("\\{}.{}", a, b.format(ctx, int)?)
                 }
             }
-            Self::Of(a, b) => format!("{} of {}", a, b.format(int)?),
-            Self::Assign(a, b) => format!("{} = {}", a, b.format(int)?),
-            Self::Statements(a, b) => format!("{}; {}", a.format(int)?, b.format(int)?),
+            Self::Of(a, b) => format!("{} of {}", a, b.format(ctx, int)?),
+            Self::Assign(a, b) => format!("{} = {}", a, b.format(ctx, int)?),
+            Self::Statements(a, b) => format!("{}; {}", a.format(ctx, int)?, b.format(ctx, int)?),
         })
     }
 }
@@ -103,7 +107,7 @@ pub(crate) fn evaluate<I: Interrupt>(
     scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value, IntErr<String, I>> {
+) -> Result<Value, FendError> {
     macro_rules! eval {
         ($e:expr) => {
             evaluate($e, scope.clone(), context, int)
@@ -116,11 +120,9 @@ pub(crate) fn evaluate<I: Interrupt>(
         Expr::Parens(x) => eval!(*x)?,
         Expr::UnaryMinus(x) => eval!(*x)?.handle_num(|x| Ok(-x), Expr::UnaryMinus, scope)?,
         Expr::UnaryPlus(x) => eval!(*x)?.handle_num(Ok, Expr::UnaryPlus, scope)?,
-        Expr::UnaryDiv(x) => eval!(*x)?.handle_num(
-            |x| Number::from(1).div(x, int).map_err(IntErr::into_string),
-            Expr::UnaryDiv,
-            scope,
-        )?,
+        Expr::UnaryDiv(x) => {
+            eval!(*x)?.handle_num(|x| Number::from(1).div(x, int), Expr::UnaryDiv, scope)?
+        }
         Expr::Factorial(x) => {
             eval!(*x)?.handle_num(|x| x.factorial(int), Expr::Factorial, scope)?
         }
@@ -143,7 +145,7 @@ pub(crate) fn evaluate<I: Interrupt>(
         }
         Expr::Mul(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
-            |a, b| a.mul(b, int).map_err(IntErr::into_string),
+            |a, b| a.mul(b, int),
             |a| |f| Expr::Mul(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Mul(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
@@ -159,14 +161,14 @@ pub(crate) fn evaluate<I: Interrupt>(
         }
         Expr::Div(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
-            |a, b| a.div(b, int).map_err(IntErr::into_string),
+            |a, b| a.div(b, int),
             |a| |f| Expr::Div(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Div(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
         )?,
         Expr::Mod(a, b) => eval!(*a)?.handle_two_nums(
             eval!(*b)?,
-            |a, b| a.modulo(b, int).map_err(IntErr::into_string),
+            |a, b| a.modulo(b, int),
             |a| |f| Expr::Mod(f, Box::new(Expr::Literal(Value::Num(Box::new(a))))),
             |a| |f| Expr::Mod(Box::new(Expr::Literal(Value::Num(Box::new(a)))), f),
             scope,
@@ -221,7 +223,7 @@ fn evaluate_add<I: Interrupt>(
     b: Value,
     scope: Option<Arc<Scope>>,
     int: &I,
-) -> Result<Value, IntErr<String, I>> {
+) -> Result<Value, FendError> {
     Ok(match (a, b) {
         (Value::Num(a), Value::Num(b)) => Value::Num(Box::new(a.add(*b, int)?)),
         (Value::String(a), Value::String(b)) => {
@@ -255,7 +257,7 @@ fn evaluate_as<I: Interrupt>(
     scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value, IntErr<String, I>> {
+) -> Result<Value, FendError> {
     if let Expr::Ident(ident) = &b {
         match ident.as_str() {
             "bool" | "boolean" => {
@@ -275,7 +277,7 @@ fn evaluate_as<I: Interrupt>(
             "string" => {
                 return Ok(Value::String(
                     evaluate(a, scope, context, int)?
-                        .format_to_plain_string(0, int)?
+                        .format_to_plain_string(0, context, int)?
                         .into(),
                 ));
             }
@@ -352,7 +354,7 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
     scope: Option<Arc<Scope>>,
     context: &mut crate::Context,
     int: &I,
-) -> Result<Value, IntErr<String, I>> {
+) -> Result<Value, FendError> {
     macro_rules! eval_box {
         ($input:expr) => {
             Box::new(evaluate_to_value($input, scope.clone(), context, int)?)
@@ -361,11 +363,8 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
     if let Some(scope) = scope.clone() {
         match scope.get(ident, context, int) {
             Ok(val) => return Ok(val),
-            Err(IntErr::Interrupt(int)) => return Err(IntErr::Interrupt(int)),
-            Err(IntErr::Error(GetIdentError::IdentifierNotFound(_))) => (),
-            Err(IntErr::Error(err @ GetIdentError::EvalError(_))) => {
-                return Err(IntErr::Error(err.to_string()))
-            }
+            Err(FendError::IdentifierNotFound(_)) => (),
+            Err(err) => return Err(err),
         }
     }
     if let Some(val) = context.variables.get(ident.as_str()) {
@@ -379,6 +378,7 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
         "i" => Value::Num(Box::new(Number::i())),
         "true" => Value::from(true),
         "false" => Value::from(false),
+        "sample" | "roll" => Value::BuiltInFunction(BuiltInFunction::Sample),
         "sqrt" => evaluate_to_value("x: x^(1/2)", scope, context, int)?,
         "cbrt" => evaluate_to_value("x: x^(1/3)", scope, context, int)?,
         "conjugate" => crate::value::func::CONJUGATE.into(),
@@ -425,7 +425,6 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
             ("mass".into(), eval_box!("5.97237e24 kg")),
             ("volume".into(), eval_box!("1.08321e12 km^3")),
         ]),
-        "differentiate" => Value::BuiltInFunction(BuiltInFunction::Differentiate),
         "today" => crate::date::Date::today(context)
             .map_err(|e| e.to_string())?
             .into(),
@@ -437,9 +436,6 @@ pub(crate) fn resolve_identifier<I: Interrupt>(
             .map_err(|e| e.to_string())?
             .prev()
             .into(),
-        _ => {
-            return crate::units::query_unit(ident.as_str(), context, int)
-                .map_err(IntErr::into_string)
-        }
+        _ => return crate::units::query_unit(ident.as_str(), context, int),
     })
 }
