@@ -6,7 +6,7 @@
 //#![forbid(clippy::cognitive_complexity)]
 #![forbid(unreachable_pub)]
 #![forbid(elided_lifetimes_in_paths)]
-#![doc(html_root_url = "https://docs.rs/fend-core/0.1.23")]
+#![doc(html_root_url = "https://docs.rs/fend-core/0.1.24")]
 
 mod ast;
 mod date;
@@ -31,6 +31,7 @@ pub use interrupt::Interrupt;
 pub struct FendResult {
     plain_result: String,
     span_result: Vec<Span>,
+    is_unit: bool, // is this the () type
 }
 
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
@@ -87,6 +88,8 @@ impl FendResult {
         self.plain_result.as_str()
     }
 
+    /// This retrieves the main result as a list of spans, which is useful
+    /// for coloured output.
     pub fn get_main_result_spans(&self) -> impl Iterator<Item = SpanRef<'_>> {
         self.span_result.iter().map(|span| SpanRef {
             string: &span.string,
@@ -94,6 +97,13 @@ impl FendResult {
         })
     }
 
+    /// Returns whether or not the result is the `()` type. It can sometimes
+    /// be useful to hide these values.
+    #[must_use]
+    pub fn is_unit_type(&self) -> bool {
+        self.is_unit
+    }
+
     /// This used to retrieve a list of other results of the computation,
     /// but now returns an empty iterator. This method is deprecated and
     /// may be removed in a future release.
@@ -224,9 +234,10 @@ pub fn evaluate_with_interrupt(
         return Ok(FendResult {
             plain_result: String::new(),
             span_result: vec![],
+            is_unit: true,
         });
     }
-    let result = match eval::evaluate_to_spans(input, None, context, int) {
+    let (result, is_unit) = match eval::evaluate_to_spans(input, None, context, int) {
         Ok(value) => value,
         Err(e) => return Err(e.to_string()),
     };
@@ -237,11 +248,48 @@ pub fn evaluate_with_interrupt(
     Ok(FendResult {
         plain_result,
         span_result: result,
+        is_unit,
     })
 }
 
+#[derive(Debug)]
+pub struct Completion {
+    display: String,
+    insert: String,
+}
+
+impl Completion {
+    #[must_use]
+    pub fn display(&self) -> &str {
+        &self.display
+    }
+
+    #[must_use]
+    pub fn insert(&self) -> &str {
+        &self.insert
+    }
+}
+
+#[must_use]
+pub fn get_completions_for_prefix(mut prefix: &str) -> (usize, Vec<Completion>) {
+    let mut prepend = "";
+    let position = prefix.len();
+    if let Some((a, b)) = prefix.rsplit_once(' ') {
+        prepend = a;
+        prefix = b;
+    }
+    if prefix.is_empty() {
+        return (0, vec![]);
+    }
+    let mut res = units::get_completions_for_prefix(prefix);
+    for c in &mut res {
+        c.display.insert_str(0, prepend);
+    }
+    (position, res)
+}
+
 const fn get_version_as_str() -> &'static str {
-    "0.1.23"
+    "0.1.24"
 }
 
 /// Returns the current version of `fend-core`.
