@@ -1,11 +1,9 @@
-use crate::error::{IntErr, Interrupt, Never};
+use crate::error::{FendError, Interrupt};
 use crate::format::Format;
 use crate::interrupt::test_int;
-use crate::num::{
-    Base, DivideByZero, Exact, IntegerPowerError, Range, RangeBound, ValueOutOfRange,
-};
+use crate::num::{out_of_range, Base, Exact, Range, RangeBound};
 use std::cmp::{max, Ordering};
-use std::fmt;
+use std::{fmt, hash};
 
 #[derive(Clone)]
 pub(crate) enum BigUint {
@@ -14,6 +12,19 @@ pub(crate) enum BigUint {
     Large(Vec<u64>),
 }
 
+impl hash::Hash for BigUint {
+    fn hash<H: hash::Hasher>(&self, state: &mut H) {
+        match self {
+            Small(u) => u.hash(state),
+            Large(v) => {
+                for u in v {
+                    u.hash(state);
+                }
+            }
+        }
+    }
+}
+
 use BigUint::{Large, Small};
 
 #[allow(clippy::as_conversions, clippy::cast_possible_truncation)]
@@ -55,13 +66,10 @@ impl BigUint {
         }
     }
 
-    pub(crate) fn try_as_usize<I: Interrupt>(
-        &self,
-        int: &I,
-    ) -> Result<usize, IntErr<ValueOutOfRange<FormattedBigUint, usize>, I>> {
+    pub(crate) fn try_as_usize<I: Interrupt>(&self, int: &I) -> Result<usize, FendError> {
         use std::convert::TryFrom;
-        let error = || -> Result<_, IntErr<Never, I>> {
-            Ok(ValueOutOfRange(
+        let error = || -> Result<_, FendError> {
+            Ok(out_of_range(
                 self.fm(int)?,
                 Range {
                     start: RangeBound::Closed(0),
@@ -75,7 +83,7 @@ impl BigUint {
                 if let Ok(res) = usize::try_from(*n) {
                     res
                 } else {
-                    return Err(error()?.into());
+                    return Err(error()?);
                 }
             }
             Large(v) => {
@@ -84,10 +92,10 @@ impl BigUint {
                     if let Ok(res) = usize::try_from(v[0]) {
                         res
                     } else {
-                        return Err(error()?.into());
+                        return Err(error()?);
                     }
                 } else {
-                    return Err(error()?.into());
+                    return Err(error()?);
                 }
             }
         })
@@ -130,7 +138,7 @@ impl BigUint {
                     // no need to do anything
                 } else {
                     self.make_large();
-                    self.set(idx, new_value)
+                    self.set(idx, new_value);
                 }
             }
             Large(value) => {
@@ -160,15 +168,9 @@ impl BigUint {
         }
     }
 
-    pub(crate) fn gcd<I: Interrupt>(
-        mut a: Self,
-        mut b: Self,
-        int: &I,
-    ) -> Result<Self, IntErr<Never, I>> {
+    pub(crate) fn gcd<I: Interrupt>(mut a: Self, mut b: Self, int: &I) -> Result<Self, FendError> {
         while b >= 1.into() {
-            let r = a
-                .rem(&b, int)
-                .map_err(|e| e.expect("unexpected division by zero"))?;
+            let r = a.rem(&b, int)?;
             a = b;
             b = r;
         }
@@ -176,29 +178,21 @@ impl BigUint {
         Ok(a)
     }
 
-    pub(crate) fn pow<I: Interrupt>(
-        a: &Self,
-        b: &Self,
-        int: &I,
-    ) -> Result<Self, IntErr<IntegerPowerError, I>> {
+    pub(crate) fn pow<I: Interrupt>(a: &Self, b: &Self, int: &I) -> Result<Self, FendError> {
         if a.is_zero() && b.is_zero() {
-            return Err(IntegerPowerError::ZeroToThePowerOfZero.into());
+            return Err(FendError::ZeroToThePowerOfZero);
         }
         if b.is_zero() {
             return Ok(Self::from(1));
         }
         if b.value_len() > 1 {
-            return Err(IntegerPowerError::ExponentTooLarge.into());
+            return Err(FendError::ExponentTooLarge);
         }
-        Ok(a.pow_internal(b.get(0), int)?)
+        a.pow_internal(b.get(0), int)
     }
 
     // computes the exact square root if possible, otherwise the next lower integer
-    pub(crate) fn root_n<I: Interrupt>(
-        self,
-        n: &Self,
-        int: &I,
-    ) -> Result<Exact<Self>, IntErr<IntegerPowerError, I>> {
+    pub(crate) fn root_n<I: Interrupt>(self, n: &Self, int: &I) -> Result<Exact<Self>, FendError> {
         if self == 0.into() || self == 1.into() || n == &Self::from(1) {
             return Ok(Exact::new(self, true));
         }
@@ -219,11 +213,7 @@ impl BigUint {
         Ok(Exact::new(low_guess, false))
     }
 
-    fn pow_internal<I: Interrupt>(
-        &self,
-        mut exponent: u64,
-        int: &I,
-    ) -> Result<Self, IntErr<Never, I>> {
+    fn pow_internal<I: Interrupt>(&self, mut exponent: u64, int: &I) -> Result<Self, FendError> {
         let mut result = Self::from(1);
         let mut base = self.clone();
         while exponent > 0 {
@@ -237,7 +227,7 @@ impl BigUint {
         Ok(result)
     }
 
-    fn lshift<I: Interrupt>(&mut self, int: &I) -> Result<(), IntErr<Never, I>> {
+    fn lshift<I: Interrupt>(&mut self, int: &I) -> Result<(), FendError> {
         match self {
             Small(n) => {
                 if *n & 0xc000_0000_0000_0000 == 0 {
@@ -262,7 +252,7 @@ impl BigUint {
         Ok(())
     }
 
-    fn rshift<I: Interrupt>(&mut self, int: &I) -> Result<(), IntErr<Never, I>> {
+    fn rshift<I: Interrupt>(&mut self, int: &I) -> Result<(), FendError> {
         match self {
             Small(n) => *n >>= 1,
             Large(value) => {
@@ -285,15 +275,15 @@ impl BigUint {
         &self,
         other: &Self,
         int: &I,
-    ) -> Result<(Self, Self), IntErr<DivideByZero, I>> {
+    ) -> Result<(Self, Self), FendError> {
         if let (Small(a), Small(b)) = (self, other) {
             if let (Some(div_res), Some(mod_res)) = (a.checked_div(*b), a.checked_rem(*b)) {
                 return Ok((Small(div_res), Small(mod_res)));
             }
-            return Err(DivideByZero {}.into());
+            return Err(FendError::DivideByZero);
         }
         if other.is_zero() {
-            return Err(DivideByZero {}.into());
+            return Err(FendError::DivideByZero);
         }
         if other == &Self::from(1) {
             return Ok((self.clone(), Self::from(0)));
@@ -332,11 +322,7 @@ impl BigUint {
     }
 
     /// computes self *= other
-    fn mul_internal<I: Interrupt>(
-        &mut self,
-        other: &Self,
-        int: &I,
-    ) -> Result<(), IntErr<Never, I>> {
+    fn mul_internal<I: Interrupt>(&mut self, other: &Self, int: &I) -> Result<(), FendError> {
         if self.is_zero() || other.is_zero() {
             *self = Self::from(0);
             return Ok(());
@@ -373,7 +359,7 @@ impl BigUint {
     }
 
     // Note: 0! = 1, 1! = 1
-    pub(crate) fn factorial<I: Interrupt>(mut self, int: &I) -> Result<Self, IntErr<Never, I>> {
+    pub(crate) fn factorial<I: Interrupt>(mut self, int: &I) -> Result<Self, FendError> {
         let mut res = Self::from(1);
         while self > 1.into() {
             test_int(int)?;
@@ -383,11 +369,7 @@ impl BigUint {
         Ok(res)
     }
 
-    pub(crate) fn mul<I: Interrupt>(
-        mut self,
-        other: &Self,
-        int: &I,
-    ) -> Result<Self, IntErr<Never, I>> {
+    pub(crate) fn mul<I: Interrupt>(mut self, other: &Self, int: &I) -> Result<Self, FendError> {
         if let (Small(a), Small(b)) = (&self, &other) {
             if let Some(res) = a.checked_mul(*b) {
                 return Ok(Self::from(res));
@@ -397,19 +379,15 @@ impl BigUint {
         Ok(self)
     }
 
-    fn rem<I: Interrupt>(&self, other: &Self, int: &I) -> Result<Self, IntErr<DivideByZero, I>> {
+    fn rem<I: Interrupt>(&self, other: &Self, int: &I) -> Result<Self, FendError> {
         Ok(self.divmod(other, int)?.1)
     }
 
-    pub(crate) fn is_even<I: Interrupt>(&self, int: &I) -> Result<bool, IntErr<DivideByZero, I>> {
+    pub(crate) fn is_even<I: Interrupt>(&self, int: &I) -> Result<bool, FendError> {
         Ok(self.divmod(&Self::from(2), int)?.1 == 0.into())
     }
 
-    pub(crate) fn div<I: Interrupt>(
-        self,
-        other: &Self,
-        int: &I,
-    ) -> Result<Self, IntErr<DivideByZero, I>> {
+    pub(crate) fn div<I: Interrupt>(self, other: &Self, int: &I) -> Result<Self, FendError> {
         Ok(self.divmod(other, int)?.0)
     }
 
@@ -550,13 +528,12 @@ impl Default for FormatOptions {
 impl Format for BigUint {
     type Params = FormatOptions;
     type Out = FormattedBigUint;
-    type Error = Never;
 
     fn format<I: Interrupt>(
         &self,
         params: &Self::Params,
         int: &I,
-    ) -> Result<Exact<Self::Out>, IntErr<Self::Error, I>> {
+    ) -> Result<Exact<Self::Out>, FendError> {
         let base_prefix = if params.write_base_prefix {
             Some(params.base)
         } else {
@@ -604,9 +581,7 @@ impl Format for BigUint {
                 let mut output = String::with_capacity(rounds);
                 while !num.is_zero() {
                     test_int(int)?;
-                    let divmod_res = num
-                        .divmod(&divisor, int)
-                        .map_err(|e| e.expect("division by zero is not allowed"))?;
+                    let divmod_res = num.divmod(&divisor, int)?;
                     let mut digit_group_value =
                         u128::from(divmod_res.1.get(1)) << 64 | u128::from(divmod_res.1.get(0));
                     for _ in 0..rounds {
@@ -699,14 +674,13 @@ impl FormattedBigUint {
 #[cfg(test)]
 mod tests {
     use super::BigUint;
-    use crate::error::{IntErr, Never};
-    type Res<E = Never> = Result<(), IntErr<E, crate::interrupt::Never>>;
+    type Res = Result<(), crate::error::FendError>;
 
     #[test]
-    fn test_sqrt() -> Res<crate::num::IntegerPowerError> {
+    fn test_sqrt() -> Res {
         let two = &BigUint::from(2);
         let int = crate::interrupt::Never::default();
-        let test_sqrt_inner = |n, expected_root, exact| -> Res<crate::num::IntegerPowerError> {
+        let test_sqrt_inner = |n, expected_root, exact| -> Res {
             let actual = BigUint::from(n).root_n(two, &int)?;
             assert_eq!(actual.value, BigUint::from(expected_root));
             assert_eq!(actual.exact, exact);
@@ -777,7 +751,7 @@ mod tests {
     }
 
     #[test]
-    fn test_small_division_by_two() -> Res<crate::num::DivideByZero> {
+    fn test_small_division_by_two() -> Res {
         let int = &crate::interrupt::Never::default();
         let two = BigUint::from(2);
         assert_eq!(BigUint::from(0).div(&two, int)?, BigUint::from(0));
@@ -793,7 +767,7 @@ mod tests {
     }
 
     #[test]
-    fn test_rem() -> Res<crate::num::DivideByZero> {
+    fn test_rem() -> Res {
         let int = &crate::interrupt::Never::default();
         let three = BigUint::from(3);
         assert_eq!(BigUint::from(20).rem(&three, int)?, BigUint::from(2));
