@@ -35,6 +35,10 @@ pub(crate) trait ValueTrait: fmt::Debug + BoxClone + 'static {
     fn apply(&self, _arg: Value) -> Option<Result<Value, String>> {
         None
     }
+
+    fn is_unit(&self) -> bool {
+        false
+    }
 }
 
 impl Clone for Box<dyn ValueTrait> {
@@ -173,6 +177,13 @@ impl Value {
         }
     }
 
+    pub(crate) fn is_unit(&self) -> bool {
+        match self {
+            Self::Dynamic(d) => d.is_unit(),
+            _ => false,
+        }
+    }
+
     pub(crate) fn handle_num(
         self,
         eval_fn: impl FnOnce(Number) -> Result<Number, FendError>,
