@@ -277,7 +277,7 @@ fn to_float_13() {
 
 #[test]
 fn to_float_14() {
-    test_eval_simple("6#0.(1) to float to base 10", "0.2");
+    test_eval_simple("6#0.(1) to float in base 10", "0.2");
 }
 
 #[test]
@@ -1623,6 +1623,11 @@ fn different_base_10() {
     test_eval("16#DEAD_BEEF", "16#deadbeef");
 }
 
+#[test]
+fn different_base_potential_dice_overlap() {
+    test_eval("16#D3AD_BEEF", "16#d3adbeef");
+}
+
 #[test]
 fn different_base_11() {
     expect_error("#", None);
@@ -2129,7 +2134,7 @@ fn units_6() {
 
 #[test]
 fn units_9() {
-    test_eval("365.25 light days -> ly", "1 ly");
+    test_eval("365.25 light days to ly", "1 ly");
 }
 
 #[test]
@@ -2164,7 +2169,7 @@ fn units_15() {
 
 #[test]
 fn units_22() {
-    test_eval("1psi -> kPa -> 5dp", "approx. 6.89475 kPa");
+    test_eval("1psi as kPa as 5dp", "approx. 6.89475 kPa");
 }
 
 #[test]
@@ -2407,7 +2412,7 @@ fn meter_to_kg_ft() {
 
 #[test]
 fn one_foot_to_inches() {
-    test_eval("1' -> inches", "12 inches");
+    test_eval("1' to inches", "12 inches");
 }
 
 #[test]
@@ -2532,12 +2537,12 @@ fn negative_mixed_fraction_subtraction() {
 
 #[test]
 fn barn_to_meters_squared() {
-    test_eval("1 barn -> m^2", "0.0000000000000000000000000001 m^2");
+    test_eval("1 barn to m^2", "0.0000000000000000000000000001 m^2");
 }
 
 #[test]
 fn liter_to_cubic_meters() {
-    test_eval("1L -> m^3", "0.001 m^3");
+    test_eval("1L to m^3", "0.001 m^3");
 }
 
 #[test]
@@ -2622,37 +2627,37 @@ fn factorial_of_three_kg() {
 
 #[test]
 fn recurring_digits_1() {
-    test_eval_simple("9/11 -> float", "0.(81)");
+    test_eval_simple("9/11 to float", "0.(81)");
 }
 
 #[test]
 fn recurring_digits_2() {
-    test_eval_simple("6#1 / 11 -> float", "6#0.(0313452421)");
+    test_eval_simple("6#1 / 11 to float", "6#0.(0313452421)");
 }
 
 #[test]
 fn recurring_digits_3() {
-    test_eval_simple("6#0 + 6#1 / 7 -> float", "6#0.(05)");
+    test_eval_simple("6#0 + 6#1 / 7 to float", "6#0.(05)");
 }
 
 #[test]
 fn recurring_digits_4() {
-    test_eval_simple("0.25 -> fraction", "1/4");
+    test_eval_simple("0.25 as fraction", "1/4");
 }
 
 #[test]
 fn recurring_digits_5() {
-    test_eval_simple("0.21 -> 1 dp", "approx. 0.2");
+    test_eval_simple("0.21 as 1 dp", "approx. 0.2");
 }
 
 #[test]
 fn recurring_digits_6() {
-    test_eval_simple("0.21 -> 1 dp -> auto", "0.21");
+    test_eval_simple("0.21 to 1 dp to auto", "0.21");
 }
 
 #[test]
 fn recurring_digits_7() {
-    test_eval_simple("502938/700 -> float", "718.48(285714)");
+    test_eval_simple("502938/700 to float", "718.48(285714)");
 }
 
 #[test]
@@ -4406,7 +4411,9 @@ fn two_km_sq_to_hectares() {
 fn kg_to_unitless() {
     expect_error(
         "kg to unitless",
-        Some("cannot convert from kg to unitless: units are incompatible"),
+        Some(
+            "cannot convert from kg to unitless: units 'kilogram' and 'unitless' are incompatible",
+        ),
     );
 }
 
@@ -4628,41 +4635,6 @@ fn minus_40_fahrenheit_to_celsius() {
     test_eval("-40 F to C", "-40 °C");
 }
 
-#[test]
-fn differentiate() {
-    test_eval("differentiate", "differentiate");
-}
-
-#[test]
-fn differentiate_zero() {
-    test_eval("differentiate 0", "0");
-}
-
-#[test]
-fn differentiate_one() {
-    test_eval("differentiate 1", "0");
-}
-
-#[test]
-fn differentiate_pi() {
-    test_eval("differentiate pi", "0");
-}
-
-#[test]
-fn differentiate_one_thousand() {
-    test_eval("differentiate 1000", "0");
-}
-
-#[test]
-fn differentiate_i() {
-    test_eval("differentiate i", "0");
-}
-
-#[test]
-fn differentiate_sin() {
-    test_eval("differentiate sin", "cos");
-}
-
 #[test]
 fn gigabits_to_gigabytes() {
     test_eval("25Gib/s to GB/s", "3.3554432 GB / s");
@@ -5376,3 +5348,70 @@ fn coulomb_farad_mode() {
         "0.000005 farad"
     );
 }
+
+#[test]
+fn test_rolling_dice() {
+    let mut ctx = Context::new();
+    ctx.set_random_u32_fn(|| 5);
+    evaluate("roll d20", &mut ctx).unwrap();
+}
+
+#[test]
+fn test_d6() {
+    test_eval_simple(
+        "d6",
+        "{ 1: 16.67%, 2: 16.67%, 3: 16.67%, 4: 16.67%, 5: 16.67%, 6: 16.67% }",
+    );
+}
+
+#[test]
+fn test_2d6() {
+    test_eval_simple("2d6", "{ 2: 2.78%, 3: 5.56%, 4: 8.33%, 5: 11.11%, 6: 13.89%, 7: 16.67%, 8: 13.89%, 9: 11.11%, 10: 8.33%, 11: 5.56%, 12: 2.78% }")
+}
+
+#[test]
+fn test_invalid_dice_syntax_1() {
+    expect_error("0d6", Some("invalid dice syntax, try e.g. `4d6`"));
+}
+
+#[test]
+fn test_invalid_dice_syntax_2() {
+    expect_error("1d0", Some("invalid dice syntax, try e.g. `4d6`"));
+}
+
+#[test]
+fn test_invalid_dice_syntax_3() {
+    expect_error("0d0", Some("invalid dice syntax, try e.g. `4d6`"));
+}
+
+#[test]
+fn test_invalid_dice_syntax_4() {
+    expect_error(
+        "d30000000000000000",
+        Some("invalid dice syntax, try e.g. `4d6`"),
+    );
+}
+
+#[test]
+fn test_invalid_dice_syntax_5() {
+    expect_error(
+        "30000000000000000d2",
+        Some("invalid dice syntax, try e.g. `4d6`"),
+    );
+}
+
+#[test]
+fn unit_literal() {
+    test_eval("()", "()");
+}
+
+#[test]
+fn empty_statements() {
+    test_eval("1234;", "1234");
+    test_eval(";432", "432");
+    test_eval(";", "()");
+    test_eval(";;3", "3");
+    test_eval("34;;;", "34");
+    test_eval(";2;;3;a=4;;4a", "16");
+    test_eval(";2;;3;a=4;;4a;;;()", "()");
+}
